<?php
namespace Teknisa\COZ\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;
use Teknisa\COZ\Services\GeracaoProdutoAcabadoService;
use Teknisa\COZ\Services\RequisicaoPlanejamentoProducaoService;

class GeracaoProdutoAcabadoController extends \Teknisa\GER\Controllers\TeknisaCrud {

    const pla07000_central = 'pla07000_central';
    const pla07000_planproducao = 'pla07000_planproducao';
    const pla07000_produtos_acabados = 'pla07000_produtos_acabados';
    const pla07000_consumo_agregada = 'pla07000_consumo_agregada';
    const pla07000_materia_prima = 'pla07000_materia_prima';
    const pla07000_receita = 'pla07000_receita';
    const pla07000_qtd_insuf_produto_gera_parcial = 'pla07000_qtd_insuf_produto_gera_parcial';
    const pla07000_geracao_parcial_por_lista = 'pla07000_geracao_parcial_por_lista';
    const pla07000_qtd_insuf_est_mp_geracao_pa = 'pla07000_qtd_insuf_est_mp_geracao_pa';

    protected $novocodigo;
    protected $connection;
    protected $entityManager;
    protected $crudService;
    protected $geracaoProdutoAcabadoService;
    protected $reqPlaProducaoService;
    protected $pla10500service;
    protected $util;
    public    $environment;
    protected $dataSourceName = "geracaoprodutoacabado";
    protected $parameterBagColumns = array(
        "NRORG"
    );
    
    public function __construct(
         $crudService, $parameterBag, $dataSourceManager, $environment,$util,
         GeracaoProdutoAcabadoService $geracaoProdutoAcabadoService, $reqPlaProducaoService,
         \Teknisa\COZ\Services\Pla10500Service $pla10500service,
         $novocodigo,$connection,$entityManager
     ) {
        $this->geracaoProdutoAcabadoService = $geracaoProdutoAcabadoService;
        $this->reqPlaProducaoService = $reqPlaProducaoService;
        $this->pla10500service = $pla10500service;
        $this->dataSourceManager = $dataSourceManager;
        $this->environment = $environment;
        $this->util = $util;
        $this->connection = $connection;
        $this->novocodigo = $novocodigo;
        $this->entityManager = $entityManager;
        $this->crudService = $crudService;
        parent::__construct($crudService, $parameterBag);
    }

    public function retornaCentral(DTO\Request\Filter $request, DTO\Response $response) {
        try {

            $filter = $request->getFilterCriteria();
            $conditions = $filter->getConditions();
//            $nrorg = $conditions[0]['value'];
            $nrorg = $this->environment->getNrOrgTrab();

            $filterCriteria = new FilterCriteria(self::pla07000_central);
            $this->parameterBagColumns = array('NRORG');
            $filterCriteria->addCondition('NRORG', $nrorg);
            $filterCriteria->setDataSourceName(self::pla07000_central);
            $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
            $dataSet = $this->dataSourceManager->findBy($newFilterCriteria);

            $response->addDataSet($dataSet);
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }

    public function retornaPlanejamentoProducao(DTO\Request\Filter $request, DTO\Response $response) {
        try {

            $filter = $request->getFilterCriteria();
            $conditions = $filter->getConditions();
//            $nrorg = $conditions[0]['value'];
            $nrorg = $this->environment->getNrOrgTrab();
            $cdfilial = $conditions[1]['value'];

            $filterCriteria = new FilterCriteria(self::pla07000_planproducao);
            $this->parameterBagColumns = array('NRORG', 'CDFILIAL');
            $filterCriteria->addCondition('NRORG', $nrorg);
            $filterCriteria->addCondition('CDFILIAL', $cdfilial);
            $filterCriteria->setDataSourceName(self::pla07000_planproducao);
            $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
            $dataSet = $this->dataSourceManager->findBy($newFilterCriteria);

            $response->addDataSet($dataSet);
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }

    public function retornaProdutosAcabados(DTO\Request\Filter $request, DTO\Response $response) {
        try {

            $filter = $request->getFilterCriteria();
            $conditions = $filter->getConditions();
//            $nrorg = $conditions[0]['value'];
            $nrorg = $this->environment->getNrOrgTrab();
            $nrplanprod = $conditions[1]['value'];

            $filterCriteria = new FilterCriteria(self::pla07000_produtos_acabados);
            $this->parameterBagColumns = array('NRORG', 'NRPLANPROD');
            $filterCriteria->addCondition('NRORG', $nrorg);
            $filterCriteria->addCondition('NRPLANPROD', $nrplanprod);
            $filterCriteria->setDataSourceName(self::pla07000_produtos_acabados);
            $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
            $dataSet = $this->dataSourceManager->findBy($newFilterCriteria);

            $response->addDataSet($dataSet);
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }

    public function retornaConsumoAgregada(DTO\Request\Filter $request, DTO\Response $response) {
        try {

            $filter = $request->getFilterCriteria();
            $conditions = $filter->getConditions();
//            $nrorg = $conditions[0]['value'];
            $nrorg = $this->environment->getNrOrgTrab();
            $nrplanprod = $conditions[1]['value'];
            $cdprodacabado = $conditions[2]['value'];
            $idcookchill = $conditions[3]['value'];
            $idestado = $conditions[4]['value'];
            $idfinalizacao = $conditions[5]['value'];
            $idtpprodchill = $conditions[6]['value'];
            $nrseqprodacabado = $conditions[7]['value'];

            $filterCriteria = new FilterCriteria(self::pla07000_consumo_agregada);
            $this->parameterBagColumns = array('NRORG', 'NRPLANPROD', 'CDPRODACABADO', 'IDCOOKCHILL', 'IDESTADO', 'IDFINALIZACAO', 'IDTPPRODCHILL', 'NRSEQPRODACABADO');
            $filterCriteria->addCondition('NRORG', $nrorg);
            $filterCriteria->addCondition('NRPLANPROD', $nrplanprod);
            $filterCriteria->addCondition('CDPRODACABADO', $cdprodacabado);
            $filterCriteria->addCondition('IDCOOKCHILL', $idcookchill);
            $filterCriteria->addCondition('IDESTADO', $idestado);
            $filterCriteria->addCondition('IDFINALIZACAO', $idfinalizacao);
            $filterCriteria->addCondition('IDTPPRODCHILL', $idtpprodchill);
            $filterCriteria->addCondition('NRSEQPRODACABADO', $nrseqprodacabado);
            $filterCriteria->setDataSourceName(self::pla07000_consumo_agregada);
            $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
            $dataSet = $this->dataSourceManager->findBy($newFilterCriteria);

            $response->addDataSet($dataSet);
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }

    public function retornaMateriaPrima(DTO\Request\Filter $request, DTO\Response $response) {
        try {

            $filter = $request->getFilterCriteria();
            $conditions = $filter->getConditions();
            $nrorg = $this->environment->getNrOrgTrab();
            $nrplanprod = $conditions[1]['value'];

            $filterCriteria = new FilterCriteria(self::pla07000_materia_prima);
            $this->parameterBagColumns = array('NRORG', 'NRPLANPROD');
            $filterCriteria->addCondition('NRORG', $nrorg);
            $filterCriteria->addCondition('NRPLANPROD', $nrplanprod);
            $filterCriteria->setDataSourceName(self::pla07000_materia_prima);
            $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
            $dataSet = $this->dataSourceManager->findBy($newFilterCriteria);

            $response->addDataSet($dataSet);
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }

    public function retornaReceita(DTO\Request\Filter $request, DTO\Response $response) {
        try {

            $filter = $request->getFilterCriteria();
            $conditions = $filter->getConditions();
//            $nrorg = $conditions[0]['value'];
            $nrorg = $this->environment->getNrOrgTrab();
            $nrplanprod = $conditions[1]['value'];
            $dtconsumo = $conditions[2]['value'];
            $cdprodacabado = $conditions[3]['value'];
            $cdfiliagreg = $conditions[4]['value'];
            $cdservfiliagreg = $conditions[5]['value'];
            $nrseqprodacabado = $conditions[6]['value'];

            $filterCriteria = new FilterCriteria(self::pla07000_receita);
            $this->parameterBagColumns = array('NRORG', 'NRPLANPROD', 'DTCONSUMO', 'CDPRODACABADO', 'CDFILIAGREG', 'CDSERVFILIAGREG', 'NRSEQPRODACABADO');
            $filterCriteria->addCondition('NRORG', $nrorg);
            $filterCriteria->addCondition('NRPLANPROD', $nrplanprod);
            $filterCriteria->addCondition('DTCONSUMO', $dtconsumo);
            $filterCriteria->addCondition('CDPRODACABADO', $cdprodacabado);
            $filterCriteria->addCondition('CDFILIAGREG', $cdfiliagreg);
            $filterCriteria->addCondition('CDSERVFILIAGREG', $cdservfiliagreg);
            $filterCriteria->addCondition('NRSEQPRODACABADO', $nrseqprodacabado);
            $filterCriteria->setDataSourceName(self::pla07000_receita);
            $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
            $dataSet = $this->dataSourceManager->findBy($newFilterCriteria);

            $response->addDataSet($dataSet);
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }

    public function geraProdutoAcabado(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $rows = $request->getRow();
//            $nrorg = $rows['NRORG'];
            $nrorg = $this->environment->getNrOrgTrab();
            $nrplanprod = $rows['NRPLANPROD'];
            $listaprodacabado = $rows['LISTAPRODACABADO'];
            $validaEstoque = $rows['VALIDAESTQ'];
            $central = $rows['CDCENTRAL'];
            $origem = $rows['ORIGEM'];
            $operador = $rows['CDOPERADOR'];
            $strProdutos = $rows['STRPRODUTO'];
            $nrlistaProdAcabado = $this->geracaoProdutoAcabadoService->montaListaProdutoAcabado($nrorg, $listaprodacabado);

            if ($validaEstoque === 'S') {
                $materiaFaltante  = $this->verificaEstoqueMPConclusaoPA(
                    $nrorg, $nrplanprod, $central, $nrlistaProdAcabado
                );
            } else {
                $materiaFaltante = array();
            }

            if (count($materiaFaltante) > 0) {
                $codigo = $this->novocodigo->getCodeByTableName(
                    "LOGMATPRIMA", $nrorg
                );

                $this->pla10500service->salvaLogMP(
                    $nrorg, $nrplanprod, $codigo,
                    $origem, $strProdutos, $materiaFaltante, $operador
                    );

                $response->addDataSet(
                    new DataSet(
                        'mensagem',
                        array('MATPRINSU' => $materiaFaltante)
                    )
                );

                return;
            }
            // Se a requisição vier da tela de painel ela verifica a gera parcial.
            if($rows['GERPAINEL'] != 'N'){
                if($rows['GERPAINEL']['GERPARCIAL'] == 'S'){
                    $qtproduzido = floatval($this->getTotalQtProduzido($rows['GERPAINEL']['PAINEL']));
                    $qtPainel = floatval(str_replace(',', '.', $rows['GERPAINEL']['PAINEL']['produzir']));
                    
                    if($qtproduzido == null) $qtproduzido = $qtPainel;
                    else if($qtproduzido < $qtPainel) $qtproduzido = $qtPainel - $qtproduzido;
                    else $qtproduzido = $qtproduzido - $qtPainel;
                    
                    $this->saveGeraPaParcial($rows['GERPAINEL']['PAINEL'], $qtproduzido, $nrorg, $operador);
                }
            }

            $retorno
                = $this->geracaoProdutoAcabadoService->geraProdutoAcabado(
                $nrorg, $nrplanprod, $nrlistaProdAcabado, $central
            );
            
            
            $response->addDataSet(
                new DataSet('mensagem', array($retorno))
            );
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }

    public function saveGeraPaParcial($rows, $qtproduzido, $nrorg, $operador){
        $dtSourceName = 'gerapaparcial';
        $sql = "INSERT INTO GERAPAPARCIAL (NRGERAPAPARCIAL, NRORG, IDATIVO, NRPLANPROD, CDPRODACABADO, IDCOOKCHILL, IDFINALIZACAO, IDTPPRODCHILL, IDESTADO, NRSEQPRODACABADO, QTPRODUZIDO, CDCONTAINER, DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU) VALUES (:NRGERAPAPARCIAL, :NRORG, :IDATIVO, :NRPLANPROD, :CDPRODACABADO, :IDCOOKCHILL, :IDFINALIZACAO, :IDTPPRODCHILL, :IDESTADO, :NRSEQPRODACABADO, :QTPRODUZIDO, :CDCONTAINER, SYSDATE, SYSDATE, :NRORGINCLUSAO, :CDOPERINCLUSAO, :NRORGULTATU, :CDOPERULTATU)";
        $cdGeraPa = $this->novocodigo->getCodeByTableName('GERAPAPARCIAL', $nrorg);
        
        $params =  array(
            'NRGERAPAPARCIAL' => $cdGeraPa,
            'NRORG'           => $nrorg,
            'IDATIVO'         => 'S',
            'NRPLANPROD'      => $rows['nrplanprod'],
            'CDPRODACABADO'   => $rows['cdprodacabado'],
            'IDCOOKCHILL'     => $rows['idcookchill'],
            'IDFINALIZACAO'   => $rows['idfinalizacao'],
            'IDTPPRODCHILL'   => $rows['idtpprodchill'],
            'IDESTADO'        => $rows['idestado'],
            'NRSEQPRODACABADO'=> $rows['nrseqprodacabado'],
            'QTPRODUZIDO'     => $qtproduzido,
            'CDCONTAINER'     => null,
            'NRORGINCLUSAO'   => $nrorg,
            'CDOPERINCLUSAO'  => $operador,
            'NRORGULTATU'     => $nrorg,
            'CDOPERULTATU'    => $operador
        );

        $stmt = $this->entityManager->getConnection()->prepare($sql);
        $stmt->execute($params);
    }

    public function getTotalQtProduzido($rows){
        $sql = "SELECT SUM(QTPRODUZIDO) AS QTPRODUZIDO FROM GERAPAPARCIAL WHERE NRPLANPROD = :NRPLANPROD AND CDPRODACABADO = :CDPRODACABADO AND IDCOOKCHILL = :IDCOOKCHILL AND IDFINALIZACAO = :IDFINALIZACAO AND IDTPPRODCHILL = :IDTPPRODCHILL AND IDESTADO = :IDESTADO AND NRSEQPRODACABADO = :NRSEQPRODACABADO";
        $params = array(
            'NRPLANPROD'      => $rows['nrplanprod'],
            'CDPRODACABADO'   => $rows['cdprodacabado'],
            'IDCOOKCHILL'     => $rows['idcookchill'],
            'IDFINALIZACAO'   => $rows['idfinalizacao'],
            'IDTPPRODCHILL'   => $rows['idtpprodchill'],
            'IDESTADO'        => $rows['idestado'],
            'NRSEQPRODACABADO'=> $rows['nrseqprodacabado']
        );
        return $this->connection->fetchColumn($sql, $params);
    }

    public function verificaEstoqueMPConclusaoPA($nrorg, $nrplanprod, $central, $nrlistaProdAcabado) {
        try {
            $filterCriteria = new FilterCriteria(self::pla07000_qtd_insuf_est_mp_geracao_pa);
            $this->parameterBagColumns = array('NRORG', 'NRPLANPROD', 'CDCENTRAL', 'LISTAPROD');
            $filterCriteria->addCondition('NRORG', $nrorg);
            $filterCriteria->addCondition('NRPLANPROD', $nrplanprod);
            $filterCriteria->addCondition('CDCENTRAL', $central);
            $filterCriteria->addCondition('LISTAPROD', $nrlistaProdAcabado);
            $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
            $dataSet = $this->dataSourceManager->findBy($newFilterCriteria);

            return $dataSet->getRows();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function retornaProdAcabadoQtInsuf($nrorg, $nrplanprod, $nrlistaProdAcabado) {
        try {
            $filterCriteria = new FilterCriteria(self::pla07000_qtd_insuf_produto_gera_parcial);
            $this->parameterBagColumns = array('NRORG', 'NRPLANPROD', 'NRLISTA');
            $filterCriteria->addCondition('NRORG', $nrorg);
            $filterCriteria->addCondition('NRPLANPROD', $nrplanprod);
            $filterCriteria->addCondition('NRLISTA', $nrlistaProdAcabado);
            $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
            $dataSet = $this->dataSourceManager->findBy($newFilterCriteria);

            return $dataSet->getRows();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function retornaGeracaoParcialPorLista($nrorg, $nrplanprod, $nrlistaProdAcabado) {
        try {
            $filterCriteria = new FilterCriteria(self::pla07000_geracao_parcial_por_lista);
            $this->parameterBagColumns = array('NRORG', 'NRPLANPROD', 'NRLISTA');
            $filterCriteria->addCondition('NRORG', $nrorg);
            $filterCriteria->addCondition('NRPLANPROD', $nrplanprod);
            $filterCriteria->addCondition('NRLISTA', $nrlistaProdAcabado);
            $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
            $dataSet = $this->dataSourceManager->findBy($newFilterCriteria);

            return $dataSet->getRows();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function verificaProdGeradoParcial(DTO\Request\Row $request, DTO\Response $response){
        try {
            $rows = $request->getRow();
            $nrorg = $this->environment->getNrOrgTrab();
            $nrplanprod = $rows['NRPLANPROD'];
            $listaprodacabado = $rows['LISTAPRODACABADO'];

            $nrlistaProdAcabado = $this->geracaoProdutoAcabadoService->montaListaProdutoAcabado($nrorg ,$listaprodacabado);
            $produtosGeradosParcial = $this->retornaGeracaoParcialPorLista($nrorg, $nrplanprod, $nrlistaProdAcabado);

            if (count($produtosGeradosParcial) > 0) {
                $response->addDataSet(new DataSet('mensagem', array('TEMPROD' => true, 'LISTAPROD' => $nrlistaProdAcabado, 'PRODGERPARC' => $produtosGeradosParcial)));
            } else {
                $response->addDataSet(new DataSet('mensagem', array('TEMPROD' => false, 'LISTAPROD' => $nrlistaProdAcabado)));
            }
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }

    public function cancelaGeracaoProdutoAcabado(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $rows = $request->getRow();
//            $nrorg = $rows['NRORG'];
            $nrorg = $this->environment->getNrOrgTrab();
            $nrplanprod = $rows['NRPLANPROD'];
            $nrlistaProdAcabado = $rows['LISTAPRODACABADO'];
            $excluiParcial = $rows['EXCLUIPARCIAL'];

            $retorno
                = $this->geracaoProdutoAcabadoService->cancelaGeracaoProdutoAcabado(
                $nrorg, $nrplanprod, $nrlistaProdAcabado, $excluiParcial
            );

            $response->addDataSet(new DataSet('mensagem', array($retorno)));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }

    public function geraListaEtiquetaProdutoAcabado(DTO\Request\Row $request, DTO\Response $response){
        $rows = $request->getRow();
        $central = $rows['CDCENTRAL'];
        $dtProdIni = $rows['DTINIPROD'];
        $dtProdFim = $rows['DTFINPROD'];
        $listaProd = $rows['LISTAPROD'];
        $nrorg = $this->environment->getNrOrgTrab();

        $filterCriteria = new FilterCriteria('pla07010_lista_etiquetas');
        $this->parameterBagColumns = array('CDCENTRAL', 'DTINIPROD', 'DTFINPROD', 'LISTAPROD');
        $filterCriteria->addCondition('CDCENTRAL', $central);
        $filterCriteria->addCondition('DTINIPROD', $dtProdIni);
        $filterCriteria->addCondition('DTFINPROD', $dtProdFim);
        $filterCriteria->addCondition('LISTAPROD', $listaProd);
        $filterCriteria->setDataSourceName('pla07010_lista_etiquetas');
        $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
        $dataSet = $this->dataSourceManager->findBy($newFilterCriteria);
        $rows = $dataSet->getRows();

        $cdlista = '';
        $nrlista = '';
        $aux = 0;
        foreach ($rows as $row) {
            $qtprodacabado = $row['QTPRODACABADO'];
            $vrqttdepratemb = $row['VRQTDEPRATEMB'];
            $numEmb = ceil($qtprodacabado/$vrqttdepratemb);

            for($i = 0; $i < $numEmb; $i++){
                $cdlista .= $row['CDPRODACABADO'] . $row['CDCONTAINER'] . '_' . str_pad($i, 10, '0', STR_PAD_LEFT) . '_' .
                    str_pad($aux++, 10, '0', STR_PAD_LEFT) . '_' . $row['NRGERAPAPARCIAL'] . '-';
            }
        }
        $lista = substr($cdlista, 0, -1);
        if ($lista) {
            $nrlista = $this->reqPlaProducaoService->geraLista($nrorg, $lista, '-');
        }

        $response->addDataSet(new DataSet('NRLISTAETIQUETA', array('NRLISTA' => $nrlista)));
    }

    public function geraListaEtiquetaProdutoAcabadoParcial(DTO\Request\Row $request, DTO\Response $response){
        $rows = $request->getRow();
        $listaProd = $rows['LISTAGERPA'];
        $nrplanprod = $rows['P_NRPLANPROD'];
        $nrorg = $rows['NRORG'];

        $filterCriteria = new FilterCriteria('pla07010_lista_etiquetas_parcial');
        $this->parameterBagColumns = array('LISTAGERPA', 'P_NRPLANPROD');
        $filterCriteria->addCondition('LISTAGERPA', $listaProd);
        $filterCriteria->addCondition('P_NRPLANPROD', $nrplanprod);
        $filterCriteria->setDataSourceName('pla07010_lista_etiquetas_parcial');
        $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
        $dataSet = $this->dataSourceManager->findBy($newFilterCriteria);
        $rows = $dataSet->getRows();

        $cdlista = '';
        $nrlista = '';
        $aux = 0;
        foreach ($rows as $row) {
            $qtprodacabado = $row['QTPRODACABADO'];
            $vrqttdepratemb = $row['VRQTDEPRATEMB'];
            $numEmb = ceil($qtprodacabado/$vrqttdepratemb);

            for($i = 0; $i < $numEmb; $i++){
                $cdlista .= $row['CDPRODACABADO'] . $row['CDCONTAINER'] . '_' . str_pad($i, 10, '0', STR_PAD_LEFT) . '_' .
                    str_pad($aux++, 10, '0', STR_PAD_LEFT) . '_' . $row['NRGERAPAPARCIAL'] . '-';
            }
        }

        $lista = substr($cdlista, 0, -1);
        if ($lista) {
            $nrlista = $this->reqPlaProducaoService->geraLista($nrorg, $lista, '-');
        }

        $response->addDataSet(new DataSet('NRLISTAETIQUETA', array('NRLISTA' => $nrlista)));
    }

    public function salvaEdicaoProdutoAcabado(DTO\Request\Row $request, DTO\Response $response){
        $row = $request->getRow();
		
        $nrorg = $this->environment->getNrOrgTrab();
        $nrplanprod = $row['NRPLANPROD'];
        $cdprodacabado = $row['CDPRODACABADO'];
        $nrseqprodacabado = $row['NRSEQPRODACABADO'];
        $idcookchill = $row['IDCOOKCHILL'];
        $idfinalizacao = $row['IDFINALIZACAO'];
        $idtpprodchill = $row['IDTPPRODCHILL'];
        $idestado = $row['IDESTADO'];
        $qtextra = $row['QTEXTRA'];
        $qtperda = $row['QTPERDA'];
        $cdcontainer = $row['CDCONTAINER'];
        $qtaproduzir = $row['QTAPRODUZIR'];

        try {
            $this->geracaoProdutoAcabadoService->salvaEdicaoProdAcabado($nrorg, $nrplanprod, $cdprodacabado, $nrseqprodacabado, $idcookchill, $idfinalizacao, $idtpprodchill, $idestado, $qtextra, $qtperda, $cdcontainer);
            $planprodfiliagrega = $this->salvaEdicaoProdFiliAgrega($nrorg, $nrplanprod, $cdprodacabado, $nrseqprodacabado, $idcookchill, $idfinalizacao, $idtpprodchill, $idestado, $qtextra, $qtperda, $qtaproduzir);
            $this->salvaEdicaoRecProd($planprodfiliagrega, $nrorg, $nrplanprod, $cdprodacabado, $nrseqprodacabado, $idcookchill, $idfinalizacao, $idtpprodchill, $idestado);
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }

    public function salvaEdicaoProdFiliAgrega(
        $nrorg, $nrplanprod, $cdprodacabado, $nrseqprodacabado, $idcookchill,
        $idfinalizacao, $idtpprodchill, $idestado, $qtextra, $qtperda, $qtaproduzir
    ){
        $planprodfiliagrega = $this->geracaoProdutoAcabadoService->retornaPlanprodfiliagregaFK(
            $nrorg, $nrplanprod, $cdprodacabado, $nrseqprodacabado, $idcookchill,
            $idfinalizacao, $idtpprodchill, $idestado);

        $percExtra = $qtextra * 100 / $qtaproduzir;
        $percPerda = $qtperda * 100 / $qtaproduzir;

        foreach($planprodfiliagrega as $item){
            $qtextra = $item->getQtaproduzir() * $percExtra / 100;
            $qtperda = $item->getQtaproduzir() * $percPerda / 100;

            $this->geracaoProdutoAcabadoService->salvaEdicaoProdFiliAgrega($item, $qtextra, $qtperda);
        }

        return $planprodfiliagrega;
    }

    public function salvaEdicaoRecProd($planprodfiliagrega, $nrorg, $nrplanprod, $cdprodacabado, $nrseqprodacabado, $idcookchill, $idfinalizacao, $idtpprodchill, $idestado){
        foreach($planprodfiliagrega as $item){
            $cdfiliagreg = $item->getCdfiliagreg();
            $cdservfiliagreg = $item->getCdservfiliagreg();
            $dtconsumo = $item->getDtconsumo();
            $qtextra = $item->getQtextra();
            $qtaproduzir = $item->getQtaproduzir();

            $planrecprod = $this->geracaoProdutoAcabadoService->retornaPlanrecprodFK($nrorg, $nrplanprod, $cdprodacabado, $nrseqprodacabado, $cdfiliagreg, $cdservfiliagreg, $dtconsumo, $idcookchill, $idfinalizacao, $idtpprodchill, $idestado);

            $percExtra = $qtextra * 100 / $qtaproduzir;

            foreach($planrecprod as $receita){
                $qtextra = $receita->getQtaproduzir() * $percExtra / 100;

                $this->geracaoProdutoAcabadoService->salvaEdicaoRecProd($receita, $qtextra);
            }
        }

        return true;
    }
    
    public function geraListaProduto(DTO\Request\Row $request, DTO\Response $response) {
     try {          
          $data = $request->getRow();
          $listafiliserv = $data["FILISERV"];
          $nrlista = $this->novocodigo->getCodeByTableName('LISTA',$data["NRORG"]);
          $sql = "INSERT INTO LISTA (NRLISTA, CDLISTA) VALUES (:NRLISTA, :CDLISTA)";
          if($listafiliserv[0] == "T"){
                $listafiliserv = $this->retornaFiliserv($data["NRORG"],$data["CDFILIAL"],$data["CDCENTRAL"]);

          }
          foreach ($listafiliserv as $filiserv) {
                $stmt = $this->connection->prepare($sql);
                $stmt->execute(
                    array(
                        "NRLISTA" => $nrlista,
                        "CDLISTA" => $filiserv
                    )
                );
            }
          
            
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet ('nrLista', array($nrlista)));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }
    
    public function deletaListaProduto(DTO\Request\Row $request, DTO\Response $response) {
        try {
              $data = $request->getRow();
              $this->util->deleteLista($data["NRLISTA"]);
            
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }
    
    public function deleteProduto(DTO\Request\Row $request, DTO\Response $response){
        try{
            $rows =  $request->getRow();
            $dataSourceName = "planprodacabado";
            $this->connection->beginTransaction();
            
            foreach ($rows as $row) {
             if($row['IDCONCLUIDO'] == 'N' && $row['IDORIGEM'] == 'MANUAL'){
                 
                 $this->deleteTable($row,"gerapaparcial");  
                 $this->deleteTable($row,"planrecprod");  
                 $this->deleteTable($row,"planprodfiliagrega");
                 
                 $newRequest = new DTO\Request\Row($row,"","","");
                 $rowToDelete = $this->crudService->getRowsToDelete($newRequest,$dataSourceName);
             	 $dataSet = new DataSet($dataSourceName, $rowToDelete);
                 $this->dataSourceManager->delete($dataSet);
                 
                 
             }   
            }
            $this->connection->commit();
        }catch (\Exception $e) {
           $this->connection->rollBack(); 
           $response->setError(new Error($e->getMessage(), 500));
        }     
        
    }
    
    public function deleteUnidadeAgreg(DTO\Request\Row $request, DTO\Response $response){
        try{
            $rows =  $request->getRow()['rows'];
            $planprodacabado = $request->getRow()['planprodacabado'];
            $dataSourceName = "planprodfiliagrega";
            $this->connection->beginTransaction();
            
            foreach ($rows as $row) {
                 $row['CDFILIAL'] = $row['CDFILIAGREG'];
                 $row['CDSERVFILI'] = $row['CDSERVFILIAGREG'];
                 
                 if($row['IDCOOKCHILL'] == 'Sim')$row['IDCOOKCHILL'] = 'S';
                 else $row['IDCOOKCHILL'] = 'N';
                 
                 //delete planrecprod
                 $rowPlanRecProd = $this->rowExistentePlanRecProd($row);
                 if(!empty($rowPlanRecProd)){
                     $this->deletePlanRecProd($row);
                 }
                 
                 //delete planprodfiliagrega
                 $newRequest = new DTO\Request\Row($row,"","","");
                 $rowToDelete = $this->crudService->getRowsToDelete($newRequest,$dataSourceName);
             	 $dataSet = new DataSet($dataSourceName, $rowToDelete);
                 $this->dataSourceManager->delete($dataSet);      
                     
                 

            }
            
                 $this->updateTables($planprodacabado,[],[]);
                 
            
            $this->connection->commit();
        }catch (\Exception $e) {
           $this->connection->rollBack(); 
           $response->setError(new Error($e->getMessage(), 500));
        }     
        
    }
    
    private function deleteTable($row,$datasourcename){
            $query = "DELETE FROM $datasourcename WHERE NRPLANPROD =:NRPLANPROD AND CDPRODACABADO =:CDPRODACABADO AND NRSEQPRODACABADO =:NRSEQPRODACABADO";
            $params = array(
              'CDPRODACABADO'   => $row['CDPRODACABADO'],
              'NRPLANPROD'      => $row['NRPLANPROD'],
              'NRSEQPRODACABADO'=> $row['NRSEQPRODACABADO']
            );
        
        $this->connection->executeUpdate($query, $params);
    }
    
    public function prodAcabadoExistente($row){
        
    	$conditions = array(
        	array(
    	    	"columnName" => "NRPLANPROD",
				"operator" => "=",
				"value" => $row["NRPLANPROD"]
        	    ),
        	array(
    	    	"columnName" => "CDPRODACABADO",
				"operator" => "=",
				"value" => $row["CDPRODACABADO"]
        	    ),
        	array(
    	    	"columnName" => "IDPRODUZIR",
				"operator" => "=",
				"value" => "S"
        	    ),
        	array(
    	    	"columnName" => "IDCONCLUIDO",
				"operator" => "=",
				"value" => "N"
        	    )
        	);
        	
			$dataSet = $this->crudService->getFilteredDataSet('planprodacabado', $conditions);
            $rows = $dataSet->getRows();
            return $rows;
    }
  
    private function concatParams($row,$params){
        $params['CDFILIAL']     = $row['CDFILIAL'];
        $params['CDPRODACABADO']= $params['CDPRODPRODUCAO'];
        $params['CDFILIAGREG']  = $row['CDFILIAL'];
        $params['CDSERVFILIAGREG']= $row['CDSERVFILI'];   
        $params['CDSERVFILI']   = $row['CDSERVFILI'];
        $params['COMENSAIS']    = $row['COMENSAIS'];
        $params['NMSERVICO']    = $row['NMSERVICO'];

        return $params;
    }
   
    public function saveProduto(DTO\Request\Row $request, DTO\Response $response){
        
        try{
            $rows = $request->getRow()["rows"];
            $params = $request->getRow()["params"];
            $this->connection->beginTransaction();
            $prevRow = $this->concatParams($rows[0],$params);
            $receita = $this->retornaReceitaProdAcabado($prevRow);
            $nrSeqProd = $this->retornaNovoNrseqProd($prevRow,'planprodacabado');
            
            $rowPlanProd = $this->savePlanProdAcabado($prevRow,$nrSeqProd);
            
            $notSavedRows = [];

            foreach($rows as $row){
              $rowToSave = $this->concatParams($row,$params);
              // verifica mudança de filial e/ou serviço para a geração de nova receita
              if($prevRow["CDFILIAL"] != $rowToSave["CDFILIAL"] || $prevRow["CDSERVFILI"] != $rowToSave["CDSERVFILI"]){
                  $receita = $this->retornaReceitaProdAcabado($rowToSave);
                  $prevRow = $rowToSave;
              }
              if(!empty($receita)){
                  
                  $rowPlanProdFiliAgreg = $this->savePlanProdFiliaAgrega($rowToSave,$receita,$nrSeqProd);
                  $this->savePlanRecProd($rowToSave,$receita,$nrSeqProd);
                  $this->updateTables($rowPlanProd,$rowPlanProdFiliAgreg,$receita);
                  $this->saveCardacliPlanProd($rowPlanProdFiliAgreg,$receita);
              }else{
                  array_push($notSavedRows,$row);
              }
            }
            $this->connection->commit();
            $response->addDataSet(new DataSet('RowsNaoSalvas', $notSavedRows));
        }catch (\Exception $e) {
           $this->connection->rollBack(); 
           $response->setError(new Error($e->getMessage(), 500));
        }

    }
    
    private function savePlanProdAcabado($row,$nrSeqProd){
             
             $row['__is_new']         = true;
             $row['QTPRODACABADO']    = 0;
             $row['QTAPRODUZIR']      = $row['QTPRODACABADO'];
             $row['NRSEQPRODACABADO'] = $nrSeqProd;
             $row['IDORIGEM']         = "MANUAL";
             $row['IDPRODUZIR']       = "S";
             $row['IDCONCLUIDO']      = "N";

             $newRequest = new DTO\Request\Row($row,"","","");
             $dataSourceName = 'planprodacabado';
             $rowToSave = $this->crudService->getRowsToSave($newRequest, $dataSourceName);
             $this->crudService->save($dataSourceName,$rowToSave);
             return $row;
    }
     
    private function savePlanProdFiliaAgrega($row,$receita,$nrSeqProd){
         $row['__is_new']         = true;
         $row['CDFILIAGREG']      = $row['CDFILIAL'];
         $row['CDSERVFILIAGREG']  = $row['CDSERVFILI'];
         $row['QTPRODSERV']       = $this->retornaQntProdServ($receita,$row);
         $row['QTCOMENSSERV']     = $row['COMENSAIS'];
         $row['NRSEQPRODACABADO'] = $nrSeqProd;
         $row['QTAPRODUZIR']      = $row['QTPRODSERV'];
         $row['IDPRODUZIR']       = 'S';
         $row['IDCONCLUIDO']      = 'N';
         $newRequest = new DTO\Request\Row($row,"","","");
         $dataSourceName = 'planprodfiliagrega';
         $rowToSave = $this->crudService->getRowsToSave($newRequest, $dataSourceName);
         $this->crudService->save($dataSourceName,$rowToSave);
         return $row;
    }
    
    private function saveCardacliPlanProd($row,$receita){
        $nrcardacliplanprod = $this->novocodigo->getCodeByTableName(
                    "CARDACLIPLANPROD", $row['NRORG']
                );
        $row['__is_new']           = true;
        $row['NRCARDACLIPLANPROD'] = $nrcardacliplanprod;  
        $row['CDPRODUTO']          = $row['CDPRODACABADO'];
        $row['CDFILIAL']           = $row['CDFILIAL'];
        $row['CDSERVFILI']         = $row['CDSERVFILI'];
        $row['QTCOMESTICLI']       = $row['QTCOMENSSERV'];
        
        $newRequest = new DTO\Request\Row($row,"","","");
        $this->dataSourceName = 'cardacliplanprod';
        $rowToSave = $this->crudService->getRowsToSave($newRequest, $this->dataSourceName);
        $this->crudService->save($this->dataSourceName,$rowToSave);
    }
    
    private function savePlanRecProd($row,$receita,$nrSeqProd){
             $rowsToSave = [];
             foreach($receita as $rec){
                 $row['__is_new']           = true;
                 $row['CDSERVFILIAGREG']    = $row['CDSERVFILI'];
                 $row['CDFILIAL']           = $row['CDCENTRAL'];
                 $row['CDPRODUTO']          = $rec['CDPRODUTO'];
                 $row['QTPRODRECEPRAT']     = $rec['QTPRODUTO'] * $row['COMENSAIS'];
                 $row['QTAPRODUZIR']        = $rec['QTPRODUTO'] * $row['COMENSAIS'];
                 $row['QTLIBERADA']         = $rec['QTPRODUTO'] * $row['COMENSAIS'];
                 $row['VRUNITRECEPRAT']     = $rec['VRCUSTOUNITPROD'];
                 $row['VRTOTALRECEPRAT']    = $rec['VRCUSTOUNITPROD'] * ($rec['QTPRODUTO'] * $row['COMENSAIS']);
                 $row['NRSEQPRODACABADO']   = $nrSeqProd ;
                 $row['IDTPRECEITA']        = $rec['IDTIPORECE'];
                 if(strtotime($row['DTCONSUMO']) > strtotime($row['DTPRODUCAO'])) $row['IDADIANTADO'] = 'S';
                 else $row['IDADIANTADO'] = 'N';
                 array_push($rowsToSave,$row); 
             }
             $dataSourceName = 'planrecprod';
             $dataSet = new DataSet($dataSourceName, $rowsToSave);
             $newRequest = new DTO\Request\DataSet($dataSet,"","","");
             $dataSetToSave = $this->crudService->getRowsToSave($newRequest, $dataSourceName);
             $this->crudService->save($dataSourceName,$dataSetToSave);
    }
    
    private function updateTables($rowPlanProd,$rowPlanProdFiliAgreg,$receita){
      try{  
            
            if(!empty($rowPlanProdFiliAgreg)){
                //Update vrunitprodserv e vrtotalprodserv de planprodfiliagrega
                $sql = "UPDATE PLANPRODFILIAGREGA SET VRUNITPRODSERV = :VRTOTALPRODSERV / QTCOMENSSERV, VRTOTALPRODSERV =:VRTOTALPRODSERV 
                        WHERE CDSERVFILIAGREG =:CDSERVFILIAGREG AND CDFILIAGREG =:CDFILIAGREG AND CDPRODACABADO =:CDPRODACABADO";
                
                $params = array(
                    'CDPRODACABADO'   => $rowPlanProdFiliAgreg['CDPRODACABADO'],
                    'CDSERVFILIAGREG' => $rowPlanProdFiliAgreg['CDSERVFILIAGREG'],
                    'VRTOTALPRODSERV' => $this->somaVrTotalProd($rowPlanProdFiliAgreg,$receita),
                    'CDFILIAGREG'     => $rowPlanProdFiliAgreg['CDFILIAGREG'],
                    'QTPRODSERV'      => $this->retornaQntProdServ($receita,$rowPlanProdFiliAgreg)
                );
                $this->entityManager->getConnection()->executeUpdate($sql, $params);
                
                
            }
            //Update qtprodacabado e qtaproduzir de planprodacabado
            $dataSourceName = 'planprodacabado';
            $rowPlanProd['__is_new'] = false;
            $rowPlanProd['QTPRODACABADO'] = $this->somaQtProdAcabado($rowPlanProd);
            $rowPlanProd['QTAPRODUZIR']   = $rowPlanProd['QTPRODACABADO'];
            $newRequest = new DTO\Request\Row($rowPlanProd,"","","");
            $rowToSave = $this->crudService->getRowsToSave($newRequest, $dataSourceName);
            $this->crudService->save($dataSourceName,$rowToSave);
            
            
        
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    private function somaVrUnitProd($row){
        $result = 0;
        $sql = "SELECT SUM(VRUNITRECEPRAT) AS VRUNITPRODSERV FROM PLANRECPROD WHERE NRPLANPROD =:NRPLANPROD AND CDPRODACABADO =:CDPRODACABADO 
                AND NRSEQPRODACABADO =:NRSEQPRODACABADO AND CDFILIAGREG =:CDFILIAGREG AND CDSERVFILIAGREG =:CDSERVFILIAGREG";
        $params = array(
         "NRPLANPROD"       => $row['NRPLANPROD'],
         "CDPRODACABADO"    => $row['CDPRODACABADO'],
         "NRSEQPRODACABADO" => $row["NRSEQPRODACABADO"],
         "CDFILIAGREG"      => $row['CDFILIAGREG'],
         "CDSERVFILIAGREG"  => $row['CDSERVFILIAGREG']
        );
        $data = $this->entityManager->getConnection()->fetchAll($sql, $params);
        if(isset($data[0]['VRUNITPRODSERV']))$result = $data[0]['VRUNITPRODSERV'];
        return $result;
        
    }
    
    private function somaVrTotalProd($row,$receita){
        $result = 0;
        $sql = "SELECT SUM(VRTOTALRECEPRAT) AS VRTOTALPRODSERV FROM PLANRECPROD WHERE NRPLANPROD =:NRPLANPROD AND CDPRODACABADO =:CDPRODACABADO 
                AND CDFILIAL =:CDFILIAL AND NRSEQPRODACABADO =:NRSEQPRODACABADO AND CDFILIAGREG =:CDFILIAGREG AND CDSERVFILIAGREG =:CDSERVFILIAGREG";
        
        $params = array(
         "NRPLANPROD"       => $row['NRPLANPROD'],
         "CDPRODACABADO"    => $row['CDPRODACABADO'],
         "CDFILIAL"         => $row['CDCENTRAL'],
         "NRSEQPRODACABADO" => $row["NRSEQPRODACABADO"],
         "CDFILIAGREG"      => $row['CDFILIAGREG'],
         "CDSERVFILIAGREG"  => $row['CDSERVFILIAGREG']
        );
        $data = $this->entityManager->getConnection()->fetchAll($sql, $params);
        if(isset($data[0]['VRTOTALPRODSERV']))$result = $data[0]['VRTOTALPRODSERV'];
        return $result;
    }
    
    private function somaQtProdAcabado($row){
        $result = 0;
        $sql = "SELECT SUM(QTPRODSERV) AS QTPRODACABADO FROM PLANPRODFILIAGREGA WHERE NRPLANPROD =:NRPLANPROD 
                AND CDPRODACABADO =:CDPRODACABADO AND NRSEQPRODACABADO =:NRSEQPRODACABADO";
        $params = array(
         "NRPLANPROD"       => $row['NRPLANPROD'],
         "CDPRODACABADO"    => $row['CDPRODACABADO'],
         "NRSEQPRODACABADO" => $row["NRSEQPRODACABADO"]
        );
        $data = $this->entityManager->getConnection()->fetchAll($sql, $params);
        if(isset($data[0]['QTPRODACABADO']))$result = $data[0]['QTPRODACABADO'];
        return $result;
        
    }
  
    private function deletePlanProdFiliagrega($row){
         $newRequest = new DTO\Request\Row($row[0],"","","");
         $dataSourceName = 'planprodfiliagrega';
         $rowsToDelete = $this->crudService->getRowsToDelete($newRequest,$dataSourceName);
     	 $dataSet = new DataSet($dataSourceName, $rowsToDelete);
         $this->dataSourceManager->delete($dataSet);
    }
    
 
    private function deletePlanRecProd($row){
        $rowsToDelete = [];
        $rows = $this->rowExistentePlanRecProd($row);    
        $dataSourceName = 'planrecprod';
        
        foreach($rows as $row){
             
             $newRequest = new DTO\Request\Row($row,"","","");
             $rowToDelete = $this->crudService->getRowsToDelete($newRequest,$dataSourceName);
         	 $dataSet = new DataSet($dataSourceName, $rowToDelete);
             $this->dataSourceManager->delete($dataSet);
        }
    }
    
    public function rowExistentePlanProdFili($row){
        
    	$conditions = array(
        	array(
    	    	"columnName" => "NRPLANPROD",
				"operator" => "=",
				"value" => $row["NRPLANPROD"]
        	    ),
        	array(
    	    	"columnName" => "CDPRODACABADO",
				"operator" => "=",
				"value" => $row["CDPRODACABADO"]
        	    ),
        	array(
    	    	"columnName" => "CDFILIAGREG",
				"operator" => "=",
				"value" => $row["CDFILIAL"]
        	    ),
        	array(
    	    	"columnName" => "CDSERVFILIAGREG",
				"operator" => "=",
				"value" =>  $row["CDSERVFILI"]
        	    )
        	);
			$dataSet = $this->crudService->getFilteredDataSet('planprodfiliagrega', $conditions);
            $row = $dataSet->getRows();
            return $row;
    }
    
    public function rowExistentePlanRecProd($row){
        $conditions = array(
        	array(
    	    	"columnName" => "NRPLANPROD",
				"operator" => "=",
				"value" => $row["NRPLANPROD"]
        	    ),
        	array(
    	    	"columnName" => "CDPRODACABADO",
				"operator" => "=",
				"value" => $row["CDPRODACABADO"]
        	    ),
        	array(
    	    	"columnName" => "CDFILIAGREG",
				"operator" => "=",
				"value" => $row["CDFILIAL"]
        	    ),
        	array(
    	    	"columnName" => "CDSERVFILIAGREG",
				"operator" => "=",
				"value" =>  $row["CDSERVFILI"]
        	    ),
    	    array(
    	    	"columnName" => "IDESTADO",
				"operator" => "=",
				"value" =>  $row["IDESTADO"]
        	    )
        	);
			$dataSet = $this->crudService->getFilteredDataSet('planrecprod', $conditions);
            $row = $dataSet->getRows();
            return $row;
    }
    
    public function retornaNovoNrseqProd($row,$table){
        
        $params = array(
            'CDPRODACABADO' => $row["CDPRODACABADO"],
            'NRPLANPROD'    => $row["NRPLANPROD"]
        );
        $sql = "SELECT MAX(NRSEQPRODACABADO) AS MAXNRSEQPROD FROM $table
                WHERE NRPLANPROD =:NRPLANPROD AND CDPRODACABADO =:CDPRODACABADO";
    
        $data= $this->entityManager->getConnection()->fetchAll($sql, $params);
        $nrSeqProd = $data[0]['MAXNRSEQPROD'] + 1;
        return $nrSeqProd;
        
    }
    
    public function retornaQtProdAcabado($row){
        
        $params = array(
            "CDPRODACABADO"    => $row["CDPRODACABADO"],
            "NRPLANPROD"       => $row["NRPLANPROD"],
            "NRSEQPRODACABADO" => $row["NRSEQPRODACABADO"]
        );
        $sql = "SELECT SUM(QTPRODSERV) AS SOMAQTPRODSERV FROM PLANPRODFILIAGREGA WHERE NRPLANPROD =:NRPLANPROD 
                AND CDPRODACABADO =:CDPRODACABADO AND NRSEQPRODACABADO =: NRSEQPRODACABADO";
    
        $data= $this->entityManager->getConnection()->fetchAll($sql, $params);
        $qtProdAcabado = $data[0]['SOMAQTPRODSERV'];
        return $qtProdAcabado;
        
    }
    
    public function retornaReceitaProdAcabado($row){
        $params = array(
            'CDPRATO'       =>  $row['CDPRATO'],
            'DTCONSUMO'     =>  $row['DTCONSUMO'],
            'CDSERVFILI'    =>  $row['CDSERVFILI'],
            'CDFILIAL'      =>  $row['CDFILIAL'],
            'CDFILAUXPRAT'  =>  $row['CDFILAUXPRAT'],
            'CDCENTRAL'     =>  $row['CDCENTRAL'],
            'CDPRODPRODUCAO'=>  $row['CDPRODACABADO'],
        );
        $sql = "SELECT CDPRATO, CDFILAUXPRAT, CDPRODUTO, QTPRODUTO, QTPADRAO AS QTPORPESSOA, IDTIPORECE,
					   ROUND(NVL(CUSTO_PRODUTO(
									:CDCENTRAL, CDPRODUTO, DECODE(PF.CDTIPCUSTEST, '', '03', PF.CDTIPCUSTEST), 
									:DTCONSUMO, SYSDATE), 
								0),
						5) AS VRCUSTOUNITPROD
				  FROM TABLE(RECEITA_UTIL(:CDFILIAL, :CDSERVFILI, :CDPRATO, :CDFILAUXPRAT, '0', 'Z', :CDPRATO,0,0,'N'))
				 INNER JOIN PARAMFILIAL PF ON PF.CDFILIAL = :CDCENTRAL
				 WHERE NVL(QTPADRAO,0) > 0
				   AND (NVL(IDTPREQPROD,'P') = 'P')
				   AND CDPRODPRODUCAO = :CDPRODPRODUCAO";
    
        $receita= $this->entityManager->getConnection()->fetchAll($sql, $params);
        return $receita;
        
    }
    
    public function retornaQntProdServ ($receita,$row){
        $qntPorPessoa = $receita[0]['QTPORPESSOA'];
        $qntProdServ = $qntPorPessoa * $row['COMENSAIS'];
        return $qntProdServ;
    }
   
    public function retornaFiliserv($nrorg,$cdfiliais,$cdCentral){
        
        $concatCdfilial = join("_", $cdfiliais);
        
        $conditions = array (
			array (
				"columnName" => "NRORG",
				"operator" 	 => "=",
				"value" 	 => $nrorg
			),
			array (
				"columnName" => "CDFILIAGREGA",
				"operator" 	 => "=",
				"value" 	 => $concatCdfilial
			),
			array (
				"columnName" => "CDGRUPSERV",
				"operator" 	 => "=",
				"value" 	 => "T"
			),
			array (
				"columnName" => "CDGRUPFILI",
				"operator" 	 => "=",
				"value" 	 => "T"
			),
			array (
				"columnName" => "CDCENTRAL",
				"operator" 	 => "=",
				"value" 	 => $cdCentral
			)
		);
		$dataSet = $this->crudService->getFilteredDataSet('pla07000_retorna_filiserv', $conditions);
		$rows = $dataSet->getRows();
	    
	    $result = [];
        foreach ($rows as $row) {
            array_push($result,$row["FILISERV"]);
        }
        	
        return $result;  
    }
    
    public function retornaPAporMateriaPrima(
        DTO\Request\Filter $request, DTO\Response $response
    ) {
        try {
            $filter = $request->getFilterCriteria();
            $conditions = $filter->getConditions();

            $nrorg = $conditions[0]['value'];
            $nrplanprod = $conditions[1]['value'];
            $produto = $conditions[2]['value'];

            $filterCriteria = new FilterCriteria("pla07000_pa_por_materia_prima");
            $this->parameterBagColumns = array('NRORG', 'NRPLANPROD', 'CDPRODUTO');
            $filterCriteria->addCondition('NRORG', $nrorg);
            $filterCriteria->addCondition('NRPLANPROD', $nrplanprod);
            $filterCriteria->addCondition('CDPRODUTO', $produto);
            $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);

            $dataSet = $this->dataSourceManager->findBy($newFilterCriteria);
            $response->addDataSet($dataSet);
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }
}